﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ServiceModel.DomainServices.Hosting;
using System.ServiceModel.DomainServices.EntityFramework;
using SL5_BOARD.Web.Model;
using System.Data.Entity.Infrastructure;
using System.Data;

namespace SL5_BOARD.Web.DomainService
{
	[EnableClientAccess]
	public class SL5_BOARDDomainService : DbDomainService<SL5_BOARD_DBCONTEXT>
	{
		#region BOARD_MAIN
		public IQueryable<BOARD_MAIN> GetBoardMain()
		{
			return this.DbContext.BOARD_MAINS.OrderBy(p => p.BOARD_ORDER);
		}

		public IQueryable<BOARD_MAIN> GetBoardMainByIncludeBoardList()
		{
			var query = this.DbContext.BOARD_MAINS
									  .Include("BoardLists");
			return query.AsQueryable();
		}

		public void InsertBoardMain(BOARD_MAIN entity)
		{
			DbEntityEntry<BOARD_MAIN> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Detached))
			{
				entityEntry.State = EntityState.Added;
			}
			else
			{
				this.DbContext.BOARD_MAINS.Add(entity);
			}
		}

		public void UpdateBoardMain(BOARD_MAIN entity)
		{
			this.DbContext.BOARD_MAINS.AttachAsModified(entity, this.ChangeSet.GetOriginal(entity), this.DbContext);
		}

		public void DeleteBoardMain(BOARD_MAIN entity)
		{
			DbEntityEntry<BOARD_MAIN> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Deleted))
			{
				entityEntry.State = EntityState.Deleted;
			}
			else
			{
				this.DbContext.BOARD_MAINS.Attach(entity);
				this.DbContext.BOARD_MAINS.Remove(entity);
			}
		}
		#endregion


		#region BOARD_LIST
        /// <summary>
        /// 기본 조회 서비스
        /// </summary>
        /// <returns></returns>
		public IQueryable<BOARD_LIST> GetBoardList()
		{
			return this.DbContext.BOARD_LISTS.OrderByDescending(p => p.BOARD_LIST_IDX);
		}

        /// <summary>
        /// 조건 조회 서비스
        /// </summary>
        /// <param name="condition">조건 문자열</param>
        /// <returns></returns>
        public IQueryable<BOARD_LIST> GetBoardListByCondition(string condition)
        {
            //게시물 중 제목이나 내용에서 조건에 해당하는 게시물 반환
            var query = from list in this.DbContext.BOARD_LISTS
                        where list.LIST_CONTENT.Contains(condition)
                              || list.LIST_TITLE.Contains(condition)
                        orderby list.BOARD_LIST_IDX descending
                        select list;

            return query.AsQueryable();
        }

		public IQueryable<BOARD_LIST> GetBoardListByBoardMain(int BoardMainIDX, string condition)
		{
            //게시판번호와 조건을 가지고 쿼리
			var query = from kkk in this.DbContext.BOARD_LISTS
						where kkk.BOARD_MAIN_IDX == BoardMainIDX
                              && (kkk.LIST_CONTENT.Contains(condition)
                                  || kkk.LIST_TITLE.Contains(condition)
                                  || condition.Length == 0)
						orderby kkk.BOARD_LIST_IDX descending
						select kkk;

			return query.AsQueryable();
		}

		public void InsertBoardList(BOARD_LIST entity)
		{
			DbEntityEntry<BOARD_LIST> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Detached))
			{
				entityEntry.State = EntityState.Added;
			}
			else
			{
				this.DbContext.BOARD_LISTS.Add(entity);
			}
		}

		public void UpdateBoardList(BOARD_LIST entity)
		{
			this.DbContext.BOARD_LISTS.AttachAsModified(entity, this.ChangeSet.GetOriginal(entity), this.DbContext);
		}

		public void DeleteBoardList(BOARD_LIST entity)
		{
			DbEntityEntry<BOARD_LIST> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Deleted))
			{
				entityEntry.State = EntityState.Deleted;
			}
			else
			{
				this.DbContext.BOARD_LISTS.Attach(entity);
				this.DbContext.BOARD_LISTS.Remove(entity);
			}
		}
		#endregion


		#region BOARD_REPLY
		public IQueryable<BOARD_REPLY> GetBoardReply()
		{
			return this.DbContext.BOARD_REPLYS;
		}

		public IQueryable<BOARD_REPLY> GetBoardReplyByBoardList(int BoardListIDX)
		{
			var query = from p in this.DbContext.BOARD_REPLYS
						where p.BOARD_LIST_IDX == BoardListIDX
						orderby p.BOARD_LIST_IDX descending
						select p;

			return query.AsQueryable();
		}

		public void InsertBoardList(BOARD_REPLY entity)
		{
			DbEntityEntry<BOARD_REPLY> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Detached))
			{
				entityEntry.State = EntityState.Added;
			}
			else
			{
				this.DbContext.BOARD_REPLYS.Add(entity);
			}
		}

		public void UpdateBoardList(BOARD_REPLY entity)
		{
			this.DbContext.BOARD_REPLYS.AttachAsModified(entity, this.ChangeSet.GetOriginal(entity), this.DbContext);
		}

		public void DeleteBoardList(BOARD_REPLY entity)
		{
			DbEntityEntry<BOARD_REPLY> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Deleted))
			{
				entityEntry.State = EntityState.Deleted;
			}
			else
			{
				this.DbContext.BOARD_REPLYS.Attach(entity);
				this.DbContext.BOARD_REPLYS.Remove(entity);
			}
		}
		#endregion


		#region BOARD_ATTACH
		public IQueryable<BOARD_ATTACH> GetBoardAttach()
		{
			return this.DbContext.BOARD_ATTACHS;
		}

		public IQueryable<BOARD_ATTACH> GetBoardAttachByBoardList(int BoardListIDX)
		{
			var query = from p in this.DbContext.BOARD_ATTACHS
						where p.BOARD_LIST_IDX == BoardListIDX
						orderby p.BOARD_LIST_IDX descending
						select p;

			return query.AsQueryable();
		}

		public void InsertBoardList(BOARD_ATTACH entity)
		{
			DbEntityEntry<BOARD_ATTACH> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Detached))
			{
				entityEntry.State = EntityState.Added;
			}
			else
			{
				this.DbContext.BOARD_ATTACHS.Add(entity);
			}
		}

		public void UpdateBoardList(BOARD_ATTACH entity)
		{
			this.DbContext.BOARD_ATTACHS.AttachAsModified(entity, this.ChangeSet.GetOriginal(entity), this.DbContext);
		}

		public void DeleteBoardList(BOARD_ATTACH entity)
		{
			DbEntityEntry<BOARD_ATTACH> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Deleted))
			{
				entityEntry.State = EntityState.Deleted;
			}
			else
			{
				this.DbContext.BOARD_ATTACHS.Attach(entity);
				this.DbContext.BOARD_ATTACHS.Remove(entity);
			}
		}
		#endregion


		#region MST_MEMBER
		public IQueryable<MST_MEMBER> GetMstMember()
		{
			return this.DbContext.MST_MEMBERS;
		}

		//이메일과 패스워드를 가지고 로그인 정보 조회
		public IQueryable<MST_MEMBER> GetMstMemberByLogin(string Email, string Password)
		{
			var query = from m in this.DbContext.MST_MEMBERS
						where m.EMAIL == Email && m.PASSWORD == Password
						select m;

			return query.AsQueryable();
		}

        //조건을 입력 받아서 해당 해당 레코드만 반환한다.
        public IQueryable<MST_MEMBER> GetMstMemberByCondition(string Condition)
        {
            var query = from m in this.DbContext.MST_MEMBERS
                        where m.MEMBER_NAME.Contains(Condition) 
                              || m.EMAIL.Contains(Condition)
                              || m.NICK_NAME.Contains(Condition)
                        select m;

            return query.AsQueryable();
        }

		public void InsertBoardList(MST_MEMBER entity)
		{
			DbEntityEntry<MST_MEMBER> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Detached))
			{
				entityEntry.State = EntityState.Added;
			}
			else
			{
				this.DbContext.MST_MEMBERS.Add(entity);
			}
		}

		public void UpdateBoardList(MST_MEMBER entity)
		{
			this.DbContext.MST_MEMBERS.AttachAsModified(entity, this.ChangeSet.GetOriginal(entity), this.DbContext);
		}

		public void DeleteBoardList(MST_MEMBER entity)
		{
			DbEntityEntry<MST_MEMBER> entityEntry = this.DbContext.Entry(entity);
			if ((entityEntry.State != EntityState.Deleted))
			{
				entityEntry.State = EntityState.Deleted;
			}
			else
			{
				this.DbContext.MST_MEMBERS.Attach(entity);
				this.DbContext.MST_MEMBERS.Remove(entity);
			}
		}
		#endregion
	}
}