﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using SL5_BOARD.Web.Model;

namespace SL5_BOARD.Models
{
    [Export(typeof(BoardList))]
    public class BoardList : INotifyPropertyChanged
    {
        //const string ctDefaultParagraphOri = "<Paragraph FontSize=\"11\" FontFamily=\"Portable User Interface\" Foreground=\"#FF000000\" FontWeight=\"Normal\" FontStyle=\"Normal\" FontStretch=\"Normal\" TextAlignment=\"Left\" LineHeight=\"0\" LineStackingStrategy=\"MaxHeight\">";
        //const string ctDefaultParagraphChg = "<Paragraph >";

        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        /// <summary>
        /// Board 도메인 서비스 임포트
        /// </summary>
        [Import(typeof(BoardContext))]
        public BoardContext BoardDomainService { get; set; }

        [Import(typeof(MstMember))]
        public MstMember _mstMember { get; set; }

        ObservableCollection<BOARD_LIST> conditionData;
        /// <summary>
        /// 조건에 맞는 데이터
        /// </summary>
        public ObservableCollection<BOARD_LIST> ConditionData
        {
            get
            {
                return conditionData;
            }
            set
            {
                if (conditionData != value)
                {
                    conditionData = value;
                    OnPropertyChanged("ConditionData");
                }
            }
        }

        BOARD_LIST currentData;
        /// <summary>
        /// 현재 선택된 데이터
        /// </summary>
        public BOARD_LIST CurrentData
        {
            get
            {
                return currentData;
            }
            set
            {
                if (currentData != value)
                {
                    currentData = value;
                    OnPropertyChanged("CurrentData");
                }
            }
        }

        string messageData;
        /// <summary>
        /// 모델 메시지 데이터
        /// </summary>
        public string MessageData
        {
            get
            {
                return messageData;
            }
            private set
            {
                if (messageData != value)
                {
                    messageData = "[" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + "] " + value;
                    OnPropertyChanged("MessageData");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public BoardList()
        {
            ConditionData = new ObservableCollection<BOARD_LIST>();

            this.PropertyChanged += new PropertyChangedEventHandler(BoardList_PropertyChanged);
        }

        /// <summary>
        /// 프로퍼티 채인지 구현
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BoardList_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
        }

        /// <summary>
        /// 조건 조회
        /// </summary>
        /// <param name="condition">조건</param>
        public void Getting(string condition)
        {
            if (condition.Length == 0)
            {
                //조건이 없을때는 모두 가지고 오는 함수 호출
                BoardDomainService.Context.Load(BoardDomainService.Context.GetBoardListQuery(),
                                                LoadBehavior.RefreshCurrent,
                                                load =>
                                                {
                                                    loadComplete(load);
                                                }, true);

            }
            else
            {
                //조건이 있으면 조건 조회 함수 호출
                BoardDomainService.Context.Load(BoardDomainService.Context.GetBoardListByConditionQuery(condition),
                                                LoadBehavior.RefreshCurrent,
                                                load =>
                                                {
                                                    loadComplete(load);
                                                }, true);
            }
        }

        public void Getting(int boardMainIdx, string condition)
        {
            //조건이 없을때는 모두 가지고 오는 함수 호출
            BoardDomainService.Context.Load(BoardDomainService.Context.GetBoardListByBoardMainQuery(boardMainIdx, condition),
                                            LoadBehavior.RefreshCurrent,
                                            load =>
                                            {
                                                loadComplete(load);
                                            }, true);
        }

        /// <summary>
        /// Load 콜백 함수
        /// </summary>
        /// <param name="lo"></param>
        private void loadComplete(LoadOperation lo)
        {
            if (lo.HasError == false)
            {
                //조회 결과를 컨디션데이터로 만들고
                ConditionData = new ObservableCollection<BOARD_LIST>(lo.AllEntities.Cast<BOARD_LIST>());

                //컨디션데이터 중에 회원과 연결이 앙되어 있는 카운트를 구하고
                var count = (from list in ConditionData
                             where list.MstMember == null
                             select list).Count();

                //카운트가 0보다 크면 회원데이터를 조회한다.
                if (count > 0)
                {
                    _mstMember.Getting();
                }

                MessageData = "조회 작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "조회 작업이 실패 했습니다.";
            }
        }

        /// <summary>
        /// 추가 함수
        /// </summary>
        /// <param name="addData">BOARD_LIST</param>
        public void Adding(Object addData)
        {
            if (addData != null && addData is BOARD_LIST)
            {
                BOARD_LIST item = addData as BOARD_LIST;
                item.REG_DT = DateTime.Now;
                item.REG_IDX = 1;

                BoardDomainService.Context.BOARD_LISTs.Add(item);
                ConditionData.Add(item);
                CurrentData = item;

                MessageData = "추가 작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }
        
        /// <summary>
        /// 삭제 함수
        /// </summary>
        /// <param name="removeData">BOARD_LIST</param>
        public void Removing(Object removeData)
        {
            if (removeData != null && removeData is BOARD_LIST)
            {
                BOARD_LIST removeList = removeData as BOARD_LIST;

                BoardDomainService.Context.BOARD_LISTs.Remove(removeList);
                ConditionData.Remove(removeList);
                if (CurrentData == removeList)
                    CurrentData = null;

                //삭제 작업은 바로 저장해서 서버에 반영하는 것이 좋다..
                Saving();

                MessageData = "삭제 작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }

        /// <summary>
        /// 저장 함수
        /// </summary>
        public void Saving()
        {
            BoardDomainService.Context.SubmitChanges(submitCallBack, null);
        }

        /// <summary>
        /// 저장 콜백
        /// </summary>
        /// <param name="so"></param>
        private void submitCallBack(SubmitOperation so)
        {
            if (so.IsComplete == true)
            {
                MessageData = "저장 작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "저장 작업을 실패 했습니다.";
            }
        }

        /// <summary>
        /// 취소 함수
        /// </summary>
        public void Canceling()
        {
            BoardDomainService.Context.RejectChanges();

            ConditionData.Remove(CurrentData);

            CurrentData = null;

            MessageData = "취소 작업을 완료 했습니다.";

        }

    }
}
