﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.ServiceModel.DomainServices.Client;
using SL5_BOARD.Resources;
using SL5_BOARD.Web.Model;

namespace SL5_BOARD.Models
{
    [Export(typeof(BoardMain))]
    public class BoardMain : INotifyPropertyChanged
    {
        [Import(typeof(BoardContext))]
        public BoardContext BoardDomainService { get; set; }

        //너무 길어서 짧게 쓸려고 만들어 놓은것
        public EntitySet<BOARD_MAIN> CollectionData
        {
            get
            {
                return BoardDomainService.Context.BOARD_MAINs;
            }
        }

        //게시판 종류 목록 바인딩 하기위해
        public ObservableCollection<CBoardType> BoardTypeCol { get; set; }

        //게시판 상태 바인딩용
        string boardStateText;
        public string BoardStateText
        {
            get
            {
                return boardStateText;
            }
            set
            {
                if (boardStateText != value)
                {
                    boardStateText = value;

                    switch (boardStateText)
                    {
                        case "준비중":      //준비중
                            CurrentData.BOARD_STATE = "01";
                            break;
                        case "사용중":      //사용중
                            CurrentData.BOARD_STATE = "02";
                            break;
                        case "일시중지":    //일시중지
                            CurrentData.BOARD_STATE = "03";
                            break;
                        case "삭제":        //삭제
                            CurrentData.BOARD_STATE = "04";
                            break;
                    }
                    OnPropertyChanged("BoardStateText");
                }
            }
        }

        //현재 선택되어있는 보드메인
        BOARD_MAIN currentData;
        public BOARD_MAIN CurrentData
        {
            get
            {
                return currentData;
            }
            set
            {
                if (currentData != value)
                {
                    currentData = value;
                    OnPropertyChanged("CurrentData");
                }
            }
        }

        string messageData;
        public string MessageData 
        { 
            get
            {
                return messageData;
            }
            private set
            {
                if (messageData != value)
                {
                    messageData = "[" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + "] " + value;
                    OnPropertyChanged("MessageData");
                }
            }
        }

        int maxCount;
        public int MaxCount
        {
            get
            {
                return maxCount;
            }
            private set
            {
                if (maxCount != value)
                {
                    maxCount = value;
                    OnPropertyChanged("MaxCount");
                }
            }
        }

        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        public BoardMain()
        {
            BoardTypeCol = new ObservableCollection<CBoardType>
            {
                new CBoardType{BoardType = "NOR", BoardTypeComboText="일반게시판"},
                new CBoardType{BoardType = "PIC", BoardTypeComboText="사진게시판"},
                new CBoardType{BoardType = "ETC", BoardTypeComboText="기타게시판"}
            };

            this.PropertyChanged += new PropertyChangedEventHandler(BoardMain_PropertyChanged);
        }

        void BoardMain_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            { 
                case "CurrentData":
                    if (CurrentData != null)
                    {
                        switch (CurrentData.BOARD_STATE)
                        {
                            case "01":      //준비중
                                BoardStateText = "준비중";
                                break;
                            case "02":      //사용중
                                BoardStateText = "사용중";
                                break;
                            case "03":      //일시중지
                                BoardStateText = "일시중지";
                                break;
                            case "04":      //삭제
                                BoardStateText = "삭제";
                                break;
                        }
                    }
                    else
                    {
                        BoardStateText = "";
                    }
                    break;
            }
        }

        public void Getting()
        {
            if (BoardDomainService.Context.IsLoading == false)
            {
                BoardDomainService.Context.GetBoardMainQuery().IncludeTotalCount = true;
                BoardDomainService.Context.Load(BoardDomainService.Context.GetBoardMainQuery(), LoadBehavior.RefreshCurrent, true);
            }
            else
            {
                MessageData = "조회 중이니 잠시 후 다시 시도해 주시기 바랍니다.";
            }
        }

        public void Adding(BOARD_MAIN bm)
        {
            if (bm != null && bm is BOARD_MAIN)
            {
                bm.REG_DT = DateTime.Now;
                bm.REG_IDX = 1;
                BoardDomainService.Context.BOARD_MAINs.Add(bm);
                CurrentData = bm;
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }

        public void Removing(BOARD_MAIN bm)
        {
            if (bm != null && bm is BOARD_MAIN)
            {
                BoardDomainService.Context.BOARD_MAINs.Remove(bm);
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }

        public void Saving()
        {
            BoardDomainService.Context.SubmitChanges(submitCallBack, null);
        }

        private void submitCallBack(SubmitOperation so)
        {
            if (so.IsComplete == true)
            {
                MessageData = "작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "작업을 실패 했습니다.";
            }
        }

        public void Canceling()
        {
            BoardDomainService.Context.RejectChanges();
        }

    }
}
