﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.ServiceModel.DomainServices.Client;
using SL5_BOARD.Resources;
using SL5_BOARD.Web.Model;
using System.Linq;

namespace SL5_BOARD.Models
{
    [Export(typeof(MstMember))]
    public class MstMember : INotifyPropertyChanged
    {
        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        [Import(typeof(BoardContext))]
        public BoardContext BoardDomainService { get; set; }

        //너무 길어서 짧게 쓸려고 만들어 놓은것 - 모든 데이터를 가진다.
        public EntitySet<MST_MEMBER> CollectionData
        {
            get
            {
                return BoardDomainService.Context.MST_MEMBERs;
            }
        }

        MST_MEMBER currentData;
        /// <summary>
        /// 현재 선택되어있는 회원
        /// </summary>
        public MST_MEMBER CurrentData
        {
            get
            {
                return currentData;
            }
            set
            {
                if (currentData != value)
                {
                    currentData = value;
                    OnPropertyChanged("CurrentData");
                }
            }
        }

        ObservableCollection<MST_MEMBER> conditionData;
        /// <summary>
        /// 조건조회시 사용되는 데이터
        /// </summary>
        public ObservableCollection<MST_MEMBER> ConditionData
        {
            get
            {
                return conditionData;
            }
            set
            {
                if (conditionData != value)
                {
                    conditionData = value;
                    OnPropertyChanged("ConditionData");
                }
            }
        }
        
        MST_MEMBER loginMember;
        /// <summary>
        /// 로그인 유저 데이터 - 이건 별도로 관리한다. 메인 페이지에 연결되어있기 때문에..
        /// </summary>
        public MST_MEMBER LoginMember
        {
            get
            {
                return loginMember;
            }
            set
            {
                if (loginMember != value)
                {
                    loginMember = value;
                    OnPropertyChanged("LoginMember");
                }
            }
        }

        string messageData;
        /// <summary>
        /// 메시지 출력용
        /// </summary>
        public string MessageData
        {
            get
            {
                return messageData;
            }
            private set
            {
                if (messageData != value)
                {
                    messageData = "[" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + "] " + value;
                    OnPropertyChanged("MessageData");
                }
            }
        }

        int maxCount;
        /// <summary>
        /// 최대 회원수
        /// </summary>
        public int MaxCount
        {
            get
            {
                return maxCount;
            }
            private set
            {
                if (maxCount != value)
                {
                    maxCount = value;
                    OnPropertyChanged("MaxCount");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public MstMember()
        {
            ConditionData = new ObservableCollection<MST_MEMBER>();

            this.PropertyChanged += new PropertyChangedEventHandler(MstMember_PropertyChanged);
        }

        //현재 모델에서 발생한 프로퍼티체인지 이벤트 처리 - 다른 프로퍼티 변경작업시 사용
        void MstMember_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //switch (e.PropertyName)
            //{ 
            //}
        }

        //전체 회원 조회
        public void Getting()
        {
            BoardDomainService.Context.GetMstMemberQuery().IncludeTotalCount = true;
            //Load가 완료되면 loadComplete 함수로 콜백됨
            BoardDomainService.Context.Load(
                                            BoardDomainService.Context.GetMstMemberQuery(),
                                            LoadBehavior.RefreshCurrent,
                                            load =>
                                            {
                                                loadComplete(load);
                                            },
                                            true);
        }

        //조건으로 회원 조회
        public void Getting(string Condition)
        {
            if (Condition.Length > 0)
            {
                //조건이 있을 때는 이걸 호출하고
                BoardDomainService.Context.Load(
                                                BoardDomainService.Context.GetMstMemberByConditionQuery(Condition),
                                                LoadBehavior.RefreshCurrent,
                                                load =>
                                                {
                                                    loadComplete(load);
                                                },
                                                true);
            }
            else
            {
                //조건이 없을 대는 그냥 전체 호출
                Getting();
            }
        }

        //로그인 
        public void Login(string email, string password)
        {
            //로그인시 이메일과 비밀번호를 쿼리에 넣어서 보내야함.(쿼리도 바꿔주고)
            if (BoardDomainService.Context.IsLoading == false)
            {
                BoardDomainService.Context.Load(
                                                BoardDomainService.Context.GetMstMemberByLoginQuery(email, password),
                                                LoadBehavior.RefreshCurrent,
                                                load => 
                                                {
                                                    if (load.HasError == false)
                                                    {
                                                        MessageData = "로그인 성공";
                                                        LoginMember = load.AllEntities.Cast<MST_MEMBER>().FirstOrDefault();
                                                    }
                                                },
                                                true);
            }
            else
            {
                MessageData = "조회 중이니 잠시 후 다시 시도해 주시기 바랍니다.";
            }
        }

        //로드 완료 콜벡 함수
        protected void loadComplete(LoadOperation lo)
        {
            if (lo.HasError == false)
            {
                //작업 성공
                if (lo.TotalEntityCount > 0)
                {
                    MessageData = "작업 성공";

                    //ConditionData를 만드는 방법 2가지
                    //1번방법 : LINQ 사용
                    //반환된 모든 엔티티를 캐스트 하는 것을 쿼리로 만들고
                    var query = from l in lo.AllEntities.Cast<MST_MEMBER>()
                                select l;
                    //컨디션 데이터를 클리어하고
                    ConditionData.Clear();
                    //쿼리를 리스트로 변경하고, 컨데션 데이터에 하나씩 끄집어 넣음
                    query.ToList().ForEach(p => ConditionData.Add(p));

                    //강제로 프로퍼티 체인지 이벤트 발생 시킴
                    OnPropertyChanged("ConditionData");
                }
                else
                {
                    //카운트가 없으면 그냥 클리어만 시키고 이벤트만 발생
                    ConditionData.Clear();
                    OnPropertyChanged("ConditionData");
                    
                }
            }
            else
            {
                //실행했던 쿼리 이름을 가지고 결과 처리
                MessageData = "작업 실패";
                switch (lo.EntityQuery.QueryName)
                {
                    case "GetMstMemberByConditionQuery":
                        break;
                    default:
                        //로그인 실패
                        MessageData = "메일 주소와 비밀번호를 확인해 주시기 바랍니다.";
                        break;
                }
            }

        }

        //회원추가
        public void Adding(MST_MEMBER mm)
        {
            if (mm != null && mm is MST_MEMBER)
            {
                mm.REG_DT = DateTime.Now;
                mm.REG_IDX = 1;
                BoardDomainService.Context.MST_MEMBERs.Add(mm);
                CurrentData = mm;
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }

        //회원삭제
        public void Removing(MST_MEMBER mm)
        {
            if (mm != null && mm is MST_MEMBER)
            {
                BoardDomainService.Context.MST_MEMBERs.Remove(mm);
            }
            else
            {
                MessageData = "데이터가 없거나 형식이 다릅니다.";
            }
        }

        //회원 정보 저장
        public void Saving()
        {
            BoardDomainService.Context.SubmitChanges(submitCallBack, null);
        }

        //저장완료 콜백 함수
        private void submitCallBack(SubmitOperation so)
        {
            if (so.IsComplete == true)
            {
                MessageData = "작업을 완료 했습니다.";
            }
            else
            {
                MessageData = "작업을 실패 했습니다.";
            }
        }

        //수정 취소
        public void Canceling()
        {
            BoardDomainService.Context.RejectChanges();
        }
    }
}