﻿using System;

using System.ComponentModel;
using System.ComponentModel.Composition;

//로그인 유저 정보
namespace SL5_BOARD.PublicData
{
    /// <summary>
    /// 로그인 정보
    /// </summary>
    [Export(typeof(LoginInfo))]
    public class LoginInfo : INotifyPropertyChanged
    {
        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        int mstMemberIdx;
        /// <summary>
        /// 로그인 유저 idx
        /// </summary>
        public int MstMemberIdx
        {
            get
            {
                return mstMemberIdx;
            }
            set
            {
                if (mstMemberIdx != value)
                {
                    mstMemberIdx = value;
                    OnPropertyChanged("MstMemberIdx");
                }
            }
        }

        string eMail;
        /// <summary>
        /// 로그인 유저 Email 
        /// </summary>
        public string EMail
        {
            get
            {
                return eMail;
            }
            set
            {
                if (eMail != value)
                {
                    eMail = value;
                    OnPropertyChanged("EMail");
                }
            }
        }

        string memberName;
        /// <summary>
        /// 로그인 유저 이름
        /// </summary>
        public string MemberName
        {
            get
            {
                return memberName;
            }
            set
            {
                if (memberName != value)
                {
                    memberName = value;
                    OnPropertyChanged("MemberName");
                }
            }
        }

        string nickName;
        /// <summary>
        /// 로그인 유저 닉네임
        /// </summary>
        public string NickName
        {
            get
            {
                return nickName;
            }
            set
            {
                if (nickName != value)
                {
                    nickName = value;
                    OnPropertyChanged("NickName");
                }
            }
        }

        DateTime lastLoginDt;
        /// <summary>
        /// 로그인 일시
        /// </summary>
        public DateTime LastLoginDt
        {
            get
            {
                return lastLoginDt;
            }
            set
            {
                if (lastLoginDt != value)
                {
                    lastLoginDt = value;
                    OnPropertyChanged("LastLoginDt");
                }
            }
        }

        string memberGrade;
        /// <summary>
        /// 로그인 유저 등급
        /// </summary>
        public string MemberGrade
        {
            get
            {
                return memberGrade;
            }
            set
            {
                if (memberGrade != value)
                {
                    memberGrade = value;
                    OnPropertyChanged("MemberGrade");
                }
            }
        }

        int listCount;
        /// <summary>
        /// 게시물 카운트
        /// </summary>
        public int ListCount
        {
            get
            {
                return listCount;
            }
            set
            {
                if (listCount != value)
                {
                    listCount = value;
                    OnPropertyChanged("ListCount");
                }
            }
        }

        int attachCount;
        /// <summary>
        /// 덧글 카운트
        /// </summary>
        public int AttachCount
        {
            get
            {
                return attachCount;
            }
            set
            {
                if (attachCount != value)
                {
                    attachCount = value;
                    OnPropertyChanged("AttachCount");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public LoginInfo()
        {
            InitLoginInfo();
        }

        /// <summary>
        /// 로그인 정보 초기화
        /// </summary>
        public void InitLoginInfo()
        {
            MstMemberIdx = 0;
            EMail = "";
            MemberName = "";
            LastLoginDt = DateTime.Parse("2000-01-01");
            MemberGrade = "";
            ListCount = 0;
            AttachCount = 0;
            IsLogin = false;
        }

        bool isLogin;
        /// <summary>
        /// 로그인 여부
        /// </summary>
        public bool IsLogin
        {
            get
            {
                return isLogin;
            }
            set
            {
                if (isLogin != value)
                {
                    isLogin = value;
                    OnPropertyChanged("IsLogin");
                }
            }
        }
    }
}
