﻿using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Expression.Interactivity.Core;
using SL5_BOARD.Models;
using SL5_BOARD.Web.Model;
using SL5_BOARD.PublicData;
using System.Windows;

namespace SL5_BOARD.ViewModels
{
    [Export(typeof(BoardListViewModel))]
    public class BoardListViewModel : INotifyPropertyChanged
    {
        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        /// <summary>
        /// 보드리스트 임포트
        /// </summary>
        [Import(typeof(BoardList))]
        public BoardList BoardListData { get; set; }

        /// <summary>
        /// 보드메인 임포트 - 현재 게시판을 알기 위해~
        /// </summary>
        BoardMain boardMain;
        [Import(typeof(BoardMain))]
        public BoardMain BoardMain 
        {
            get
            {
                return boardMain;
            }
            set
            {
                if (boardMain != value)
                {
                    boardMain = value;
                    boardMain.PropertyChanged += new PropertyChangedEventHandler(boardMain_PropertyChanged);
                }

            }
        }

        /// <summary>
        /// 로그인 정보 임포트
        /// </summary>
        [Import(typeof(LoginInfo))]
        public LoginInfo LoginIF { get; set; }

        /// <summary>
        /// 수정
        /// </summary>
        bool isEditting;
        public bool IsEditting 
        {
            get 
            {
                return isEditting;
            }
            set 
            {
                if (isEditting != value)
                {
                    isEditting = value;
                    OnPropertyChanged("IsEditting");
                }
            }
        }

        /// <summary>
        /// Board_Main 프로퍼티 체인지 이벤트 사용
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void boardMain_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //현재 게시판이 변경되면 게시판 목록 초기화하고 다시 조회
            switch (e.PropertyName)
            { 
                case "CurrentData":
                    BoardListData.ConditionData.Clear();
                    BoardListData.Getting(boardMain.CurrentData.BOARD_MAIN_IDX, "");
                    break;
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public BoardListViewModel()
        {
            this.PropertyChanged += new PropertyChangedEventHandler(BoardListViewModel_PropertyChanged);
        }

        /// <summary>
        /// 프로퍼티 체인지 이벤트 구현
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BoardListViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //로컬 프로퍼티 변경시 다른 프로퍼티에 영향을 미치는 경우 코딩
        }

        ICommand selectConditionCommand;
        /// <summary>
        /// 조건 조회 커맨드
        /// </summary>
        public ICommand SelectConditionCommand
        {
            get
            {
                if (selectConditionCommand == null)
                {
                    selectConditionCommand = new ActionCommand(condition =>
                    {
                        //조건을 가지는 조회 함수 실행
                        if (BoardMain != null && BoardMain.CurrentData != null)
                        {
                            BoardListData.Getting(BoardMain.CurrentData.BOARD_MAIN_IDX, condition as string);
                        }
                    });
                }
                return selectConditionCommand;
            }
        }

        ICommand addCommand;
        /// <summary>
        /// 추가 커맨드
        /// </summary>
        public ICommand AddCommand
        {
            get
            {
                if (addCommand == null)
                {
                    addCommand = new ActionCommand(() =>
                    {
                        //글쓰기는 로그인 후에만 가능
                        if (LoginIF.IsLogin == true)
                        {
                            BOARD_LIST obj = new BOARD_LIST();
                            obj.BOARD_MAIN_IDX = BoardMain.CurrentData.BOARD_MAIN_IDX;
                            obj.MST_MEMBER_IDX = LoginIF.MstMemberIdx;
                            BoardListData.Adding(obj);

                            IsEditting = true;
                        }
                        else
                        {
                            MessageBox.Show("로그인 후 이용이 가능합니다.");
                        }
                    });
                }
                return addCommand;
            }
        }

        ICommand removeCommand;
        /// <summary>
        /// 삭제 커맨드
        /// </summary>
        public ICommand RemoveCommand
        {
            get
            {
                if (removeCommand == null)
                {
                    removeCommand = new ActionCommand(() =>
                    {
                        if (BoardListData.CurrentData != null)
                        {
                            BoardListData.Removing(BoardListData.CurrentData);
                            IsEditting = false;
                        }
                    });
                }
                return removeCommand;
            }
        }

        ICommand saveCommand;
        /// <summary>
        /// 저장 커맨드
        /// </summary>
        public ICommand SaveCommand
        {
            get
            {
                if (saveCommand == null)
                {
                    saveCommand = new ActionCommand(item =>
                    {
                        RichTextBox rtb = item as RichTextBox;
                        if (rtb != null)
                        {
                            BoardListData.CurrentData.LIST_CONTENT = rtb.Xaml;
                            BoardListData.Saving();
                        }
                        IsEditting = false;
                    });
                }
                return saveCommand;
            }
        }

        ICommand cancelCommand;
        /// <summary>
        /// 취소 커맨드
        /// </summary>
        public ICommand CancelCommand
        {
            get
            {
                if (cancelCommand == null)
                {
                    cancelCommand = new ActionCommand(() =>
                    {
                        BoardListData.Canceling();

                        IsEditting = false;
                    });
                }
                return cancelCommand;
            }
        }

        ICommand selectChanged;
        /// <summary>
        /// 현재 아이템 변경 커맨드
        /// </summary>
        public ICommand SelectChangedCommand
        {
            get
            {
                if (selectChanged == null)
                {
                    selectChanged = new ActionCommand(obj => SelectChangedOperation(obj));
                }
                return selectChanged;
            }
        }
        /// <summary>
        /// 커맨드 엑션이 발생하지 않아 이곳을 직접 호출
        /// </summary>
        /// <param name="obj"></param>
        public void SelectChangedOperation(object obj)
        {
            if (obj != null && obj is BOARD_LIST)
            {
                BoardListData.CurrentData = obj as BOARD_LIST;
            }
        }

        ICommand listCommand;
        /// <summary>
        /// 목록으로 커맨드
        /// </summary>
        public ICommand ListCommand
        {
            get
            {
                if (listCommand == null)
                {
                    listCommand = new ActionCommand(() =>
                    {
                        BoardListData.CurrentData = null;

                        IsEditting = false;
                    });
                }
                return listCommand;
            }
        }

        /// <summary>
        /// 수정시작
        /// </summary>
        ICommand editCommand;
        public ICommand EditCommand
        {
            get
            {
                if (editCommand == null)
                {
                    editCommand = new ActionCommand(() =>
                    {
                        IsEditting = true;
                    });
                }
                return editCommand;
            }
        }

    }
}
