﻿using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Windows.Input;
using Microsoft.Expression.Interactivity.Core;
using SL5_BOARD.Models;
using SL5_BOARD.Web.Model;
using System.Collections.ObjectModel;
using System.Windows.Controls;
using SL5_BOARD.PublicData;
using System;
using System.Linq;
using System.Windows;

namespace SL5_BOARD.ViewModels
{
    //메인 페이지 익스포트
    [Export(typeof(MainPageViewModel))]
    public class MainPageViewModel : INotifyPropertyChanged
    {
        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        // 보드메인 모델 임포트 - 메인 메뉴 때문에 추가된 내용
        [Import(typeof(BoardMain))]
        public BoardMain BoardMainData { get; set; }

        /// <summary>
        /// 로그인 처리를 하기위한 멤버마스터 모델 임포트
        /// </summary>
        MstMember mstMemberModel;
        [Import(typeof(MstMember))]
        public MstMember MstMemberModel
        {
            get
            {
                return mstMemberModel;
            }
            set
            {
                mstMemberModel = value;
                mstMemberModel.PropertyChanged += new PropertyChangedEventHandler(MstMemberModel_PropertyChanged);
            }
        }

        /// <summary>
        /// 멤버 마스터 모델의 프로퍼티 체인지 이벤트 구현
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MstMemberModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            { 
                case "MessageData":
                    //모델에서 발생한 메시지를 뷰모델로 끌어 올려서 통합 적으로 관리
                    MessageData = MstMemberModel.MessageData;
                    break;
                case "LoginMember":
                    if (MstMemberModel.LoginMember != null)
                    {
                        //로그인 성공했을 경우
                        // admin@localhost
                        // p@ssw0rd
                        MST_MEMBER member = MstMemberModel.LoginMember;

                        LoginIF.MstMemberIdx = member.MST_MEMBER_IDX;
                        LoginIF.EMail = member.EMAIL;
                        LoginIF.MemberName = member.MEMBER_NAME;
                        LoginIF.NickName = member.NICK_NAME;
                        LoginIF.MemberGrade = member.MEMBER_GRADE;

                        LoginIF.ListCount = member.BoardLists.Count;
                        LoginIF.LastLoginDt = DateTime.Now;
                        member.LAST_LOGIN_DT = DateTime.Now;
                        LoginIF.IsLogin = true;

                        //마지막 로그인 일자 저장
                        mstMemberModel.Saving();
                    }
                    else
                    { 
                        //로그아웃 null값을 가지는 경우는 강제로 넣는 것 말고는 없냉
                        MessageData = "로그아웃 되었습니다.";
                        LoginEmail = "";
                        LoginPassword = "";

                        LoginIF.MstMemberIdx = 0;
                        LoginIF.EMail = "";
                        LoginIF.MemberName = "";
                        LoginIF.NickName = "";
                        LoginIF.MemberGrade = "";
                        LoginIF.ListCount = 0;
                        LoginIF.LastLoginDt = DateTime.Parse("2000-01-01");
                        LoginIF.IsLogin = false;
                    }
                    break;
            }
        }

        /// <summary>
        /// 메시지 출력용
        /// </summary>
        string messageData;
        public string MessageData
        {
            get
            {
                return messageData;
            }
            private set
            {
                if (messageData != value)
                {
                    messageData = value;
                    OnPropertyChanged("MessageData");
                }
            }
        }

        /// <summary>
        /// 보드리스트 모델 임포트 - 선택된 게시물 변경
        /// </summary>
        BoardList boardListData;
        [Import(typeof(BoardList))]
        public BoardList BoardListData 
        {
            get
            {
                return boardListData;
            }
            set 
            {
                boardListData = value;
                BoardListData.PropertyChanged += new PropertyChangedEventHandler(BoardListData_PropertyChanged);
            }
        }

        /// <summary>
        /// 보드리스트 데이터 모델 프로퍼티 체인지 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BoardListData_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "CurrentData":
                    if (BoardListData.CurrentData != null)
                    {
                        CurrentViewName = "BoardListDetailView";
                    }
                    else
                    {
                        CurrentViewName = "BoardListView";
                    }
                    break;
            }
        }

        /// <summary>
        /// 로그인 정보 임포트
        /// </summary>
        [Import(typeof(LoginInfo))]
        public LoginInfo LoginIF { get; set; }

        string loginEmail;
        /// <summary>
        /// 로그인 유저 이메일
        /// </summary>
        public string LoginEmail
        {
            get
            {
                return loginEmail;
            }
            set
            {
                if (loginEmail != value)
                {
                    loginEmail = value;
                    OnPropertyChanged("LoginEmail");
                }
            }
        }

        string loginPassword;
        /// <summary>
        /// 로그인 유저 비밀번호
        /// </summary>
        public string LoginPassword
        {
            get
            {
                return loginPassword;
            }
            set
            {
                if (loginPassword != value)
                {
                    loginPassword = value;
                    OnPropertyChanged("LoginPassword");
                }
            }
        }


        UserControl currentView;
        /// <summary>
        /// 현재 컨텐츠 뷰에 보이는 뷰 - 뷰를 뷰모델에서 만들어서 쑤셔 넣는 것이 마음에 들지 않아서 여기서는 않함
        /// </summary>
        public UserControl CurrentView
        {
            get
            {
                return currentView;
            }
            set
            {
                if (currentView != value)
                {
                    currentView = value;
                    OnPropertyChanged("CurrentView");
                }
            }
        }


        string currentViewName;
        /// <summary>
        /// 현재 컨텐츠 뷰 이름 - 이름을 바꾸면 프로퍼티 체인지 이벤트를 타고 메인 뷰로 올라간다
        /// </summary>
        public string CurrentViewName
        {
            get
            {
                return currentViewName;
            }
            set
            {
                if (currentViewName != value)
                {
                    currentViewName = value;
                    OnPropertyChanged("CurrentViewName");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPageViewModel()
        {
            if (!DesignerProperties.IsInDesignTool)
            {
                //로그인 인포 임시 설정
                //LoginIF.MstMemberIdx = 1;
                //LoginIF.EMail = "admin@localhost";
                //LoginIF.MemberName = "카키104";
                //LoginIF.NickName = "관리자";
                //LoginIF.LastLoginDt = DateTime.Now;
                //LoginIF.MemberGrade = "관리자등급";
                //LoginIF.ListCount = 1111;
                //LoginIF.AttachCount = 2222;
            }
        }

        //메인 메뉴 조회
        public void MenuSelect()
        {
            BoardMainData.Getting();
        }

        ICommand menuChanged;
        /// <summary>
        /// 메인 메뉴 체인지 커맨드
        /// </summary>
        public ICommand MenuChangedCommand
        {
            get
            {
                if (menuChanged == null)
                {
                    menuChanged = new ActionCommand(para => MenuChangedOperation(para));
                }
                return menuChanged;
            }
        }
        /// <summary>
        /// 메인메뉴 체인지 오퍼레이션
        /// </summary>
        /// <param name="para"></param>
        private void MenuChangedOperation(object para)
        {
            //현재 메인 메뉴 변경
            BoardMainData.CurrentData = para as BOARD_MAIN;
            CurrentViewName = "BoardListView";
        }

        ICommand selectCommand;
        /// <summary>
        /// 폼 로드시 실행 - 전체 메뉴 조회
        /// </summary>
        public ICommand SelectCommand
        {
            get
            {
                if (selectCommand == null)
                {
                    selectCommand = new ActionCommand(() =>
                    {
                        BoardMainData.Getting();
                        //처음 화면으로 게시판 관리 화면 출력
                        CurrentViewName = "BoardMainView";
                    });
                }
                return selectCommand;
            }
        }

        ICommand boardMain;
        /// <summary>
        /// 게시판 관리 커맨드
        /// </summary>
        public ICommand BoardMainCommand
        {
            get
            {
                if (boardMain == null)
                {
                    boardMain = new ActionCommand(() =>
                    {
                        CurrentViewName = "BoardMainView";
                    });
                }
                return boardMain;
            }
        }

        ICommand mstMember;
        /// <summary>
        /// 회원 관리 커맨드
        /// </summary>
        public ICommand MstMemberCommand
        {
            get
            {
                if (mstMember == null)
                {
                    mstMember = new ActionCommand(() =>
                    {
                        CurrentViewName = "MstMemberView";
                    });
                }
                return mstMember;
            }
        }


        ICommand loginCommand;
        public ICommand LoginCommand
        {
            get
            {
                if (loginCommand == null)
                {
                    loginCommand = new ActionCommand(() =>
                    {
                        //로그인 확인
                        MstMemberModel.Login(LoginEmail, LoginPassword);
                    });
                }
                return loginCommand;
            }
        }

        ICommand logoutCommand;
        public ICommand LogoutCommand
        {
            get
            {
                if (logoutCommand == null)
                {
                    logoutCommand = new ActionCommand(() =>
                    {
                        //로그아웃
                        MstMemberModel.LoginMember = null;
                    });
                }
                return logoutCommand;
            }
        }
    }
}
