﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel.Composition;
using SL5_BOARD.ViewModels;
using System.ComponentModel;

namespace SL5_BOARD.Views
{
    public partial class BoardListDetailView : UserControl
    {
        /// <summary>
        /// BoardListViewModel Import~
        /// </summary>
        [Import(typeof(BoardListViewModel))]
        public BoardListViewModel BoardListVM
        {
            get
            {
                return this.DataContext as BoardListViewModel;
            }
            set
            {
                //임포트 하자마자 데이터컨텍스트에 연결한다.
                this.DataContext = value;
                //BoardListData의 프로퍼티 체인지 이벤트 구현
                value.BoardListData.PropertyChanged += new PropertyChangedEventHandler(BoardListData_PropertyChanged);
            }
        }

        /// <summary>
        /// BoardListData의 프로퍼티 체인지 이벤트 구현
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void BoardListData_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "CurrentData":
                    if (BoardListVM.BoardListData.CurrentData != null)
                    {
                        //리치텍스트박스 xaml에 데이터 넣기
                        if (BoardListVM.BoardListData.CurrentData.LIST_CONTENT != null)
                            rtb.Xaml = BoardListVM.BoardListData.CurrentData.LIST_CONTENT;
                        else
                            rtb.Blocks.Clear();
                    }
                    break;
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public BoardListDetailView()
        {
            InitializeComponent();

            CompositionInitializer.SatisfyImports(this);

            //LayoutRoot.DataContext = BoardListVM;

        }

        #region 뷰모델에 넣지 않는다

        //  Callback for returning focus to RichTextBox
        private void ReturnFocus()
        {
            if (rtb != null)
                rtb.Focus();
        }

        /// <summary>
        /// 붙여넣기 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPaste_Click(object sender, RoutedEventArgs e)
        {
            rtb.Selection.Text = Clipboard.GetText();
            ReturnFocus();
        }

        /// <summary>
        /// 잘라내기 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCut_Click(object sender, RoutedEventArgs e)
        {
            Clipboard.SetText(rtb.Selection.Text);
            rtb.Selection.Text = "";
            ReturnFocus();
        }

        /// <summary>
        /// 복사하기 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCopy_Click(object sender, RoutedEventArgs e)
        {
            Clipboard.SetText(rtb.Selection.Text);
            ReturnFocus();
        }

        /// <summary>
        /// 폰트 변경 콤보
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbFonts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                rtb.Selection.ApplyPropertyValue(Run.FontFamilyProperty,
                    new FontFamily((cmbFonts.SelectedItem as ComboBoxItem).Tag.ToString()));
            }
            ReturnFocus();
        }

        /// <summary>
        /// 폰트 크기 변경 콤보
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbFontSizes_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                rtb.Selection.ApplyPropertyValue(Run.FontSizeProperty,
                    double.Parse((cmbFontSizes.SelectedItem as ComboBoxItem).Tag.ToString()));
            }
            ReturnFocus();
        }

        /// <summary>
        /// 굵게 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBold_Click(object sender, RoutedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                if (rtb.Selection.GetPropertyValue(Run.FontWeightProperty) is FontWeight &&
                    ((FontWeight)rtb.Selection.GetPropertyValue(Run.FontWeightProperty)) == FontWeights.Normal)
                    rtb.Selection.ApplyPropertyValue(Run.FontWeightProperty, FontWeights.Bold);
                else
                    rtb.Selection.ApplyPropertyValue(Run.FontWeightProperty, FontWeights.Normal);

            }
            ReturnFocus();
        }

        /// <summary>
        /// 이탤릭 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnItalic_Click(object sender, RoutedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                if (rtb.Selection.GetPropertyValue(Run.FontStyleProperty) is FontStyle &&
                    ((FontStyle)rtb.Selection.GetPropertyValue(Run.FontStyleProperty)) == FontStyles.Normal)
                    rtb.Selection.ApplyPropertyValue(Run.FontStyleProperty, FontStyles.Italic);
                else
                    rtb.Selection.ApplyPropertyValue(Run.FontStyleProperty, FontStyles.Normal);
            }
            ReturnFocus();
        }

        /// <summary>
        /// 밑줄 버튼
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUnderline_Click(object sender, RoutedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                if (rtb.Selection.GetPropertyValue(Run.TextDecorationsProperty) == null)
                    rtb.Selection.ApplyPropertyValue(Run.TextDecorationsProperty, TextDecorations.Underline);
                else
                    rtb.Selection.ApplyPropertyValue(Run.TextDecorationsProperty, null);
            }
            ReturnFocus();
        }

        /// <summary>
        /// 글씨색 변경 콤보
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbFontColors_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (rtb != null && rtb.Selection.Text.Length > 0)
            {
                string color = (cmbFontColors.SelectedItem as ComboBoxItem).Tag.ToString();

                SolidColorBrush brush = new SolidColorBrush(Color.FromArgb(
                    byte.Parse(color.Substring(0, 2), System.Globalization.NumberStyles.HexNumber),
                    byte.Parse(color.Substring(2, 2), System.Globalization.NumberStyles.HexNumber),
                    byte.Parse(color.Substring(4, 2), System.Globalization.NumberStyles.HexNumber),
                    byte.Parse(color.Substring(6, 2), System.Globalization.NumberStyles.HexNumber)));

                rtb.Selection.ApplyPropertyValue(Run.ForegroundProperty, brush);
            }
            ReturnFocus();
        }

        private void btnImage_Click(object sender, RoutedEventArgs e)
        {
            //InlineUIContainer container = new InlineUIContainer();
            //container.Child = getImage();
            //rtb.Selection.Insert(container);
            //ReturnFocus();
        }

        private void btnHyperlink_Click(object sender, RoutedEventArgs e)
        {
            //InsertURL cw = new InsertURL(rtb.Selection.Text);
            //cw.HasCloseButton = false;
            //cw.Closed += (s, args) =>
            //{
            //    if (cw.DialogResult.Value)
            //    {
            //        Hyperlink hyperlink = new Hyperlink();
            //        hyperlink.TargetName = "_blank";
            //        hyperlink.NavigateUri = new Uri(cw.txtURL.Text);

            //        if (cw.txtURLDesc.Text.Length > 0)
            //            hyperlink.Inlines.Add(cw.txtURLDesc.Text);
            //        else
            //            hyperlink.Inlines.Add(cw.txtURL.Text);

            //        rtb.Selection.Insert(hyperlink);
            //    }
            //};

            //cw.Show();

        }

        private void rtb_SelectionChanged(object sender, RoutedEventArgs e)
        {
        }

        private void btnSave_Click(object sender, System.Windows.RoutedEventArgs e)
        {
        }

        #endregion

    }
}
